from UserDict import UserDict
from AccessControl.SpecialUsers import nobody
from OFS.SimpleItem import SimpleItem
from OFS.Folder import Folder


class MailHostMock(object):

    def __init__(self):
        self.emails = []

    def send(self, *args, **kwargs):
        self.emails.append((args, kwargs))


class MembershipToolMock(SimpleItem):
    """
    mock portal_membership as needed
    """
    id = 'portal_membership'

    def __init__(self):
        self.members = {}
        self.authenticated = None

    def getMemberById(self, memberid):
        return self.members.get(memberid)

    def getAuthenticatedMember(self):
        return self.members.get(self.authenticated, nobody)


class MemberdataToolMock(SimpleItem):
    """
    ... portal_memberdata ...
    """
    id = 'portal_memberdata'

    def __init__(self, property_ids=('email', 'firstname', 'lastname', 'salutation')):
        self.property_ids = property_ids

    def propertyIds(self):
        return self.property_ids


class MemberDataMock(UserDict):

    getProperty = UserDict.__getitem__

    def __hash__(self):
        return hash(self['id'])

    def getId(self):
        return self['id']


class ContentMock(SimpleItem):
    """
    Dublin Core as needed
    """

    def __init__(self, id, title, creator=None):
        self.id = id
        self.title = title
        self.creator = creator

    def Creator(self):
        return self.creator

    def Title(self):
        return self.title

    def absolute_url(self):
        return 'http://nohost/%s' % self.id


class ForumMock(ContentMock, Folder):
    
    def __init__(self, id, title):
        ContentMock.__init__(self, id, title)
        Folder.__init__(self, id)

    def getPhysicalPath(self):
        return (self.id,)


class ConversationMock(ContentMock, Folder):
    """
    Mock relevant interface of PloneboardConversation
    """

    def __init__(self, id, title, forum, creator=None, commenttext=''):
        Folder.__init__(self, id)
        ContentMock.__init__(self, id, title, creator)
        self.forum = forum
        self.comments = []
        comment = CommentMock(id, title, self, creator, commenttext)
        self.comments.append(comment)
        comment = comment.__of__(self)
        self._setObject(id, comment)

    def getForum(self):
        return self.forum

    def absolute_url(self):
        return '%s/%s' % (self.forum.absolute_url(), self.id)

    def getComments(self):
        return [self._getOb(comment.id) for comment in self.comments]

    def getPhysicalPath(self):
        return self.getForum().getPhysicalPath() + (self.id,)


class CommentMock(ContentMock):
    """
    Mock relevant interface of PloneboardComment
    """

    def __init__(self, id, title, conversation, creator=None, text=''):
        super(CommentMock, self).__init__(id, title, creator)
        self.conversation = conversation
        self.text = text

    def getConversation(self):
        return self.conversation

    def getText(self):
        return self.text

    def absolute_url(self):
        return '%s/%s' % (self.conversation.absolute_url(), self.id)




import logging
from persistent.mapping import PersistentMapping
from persistent.list import PersistentList
from zope.interface import implements
from Products.CMFCore.utils import getToolByName
from .interfaces import ISubscriptions

log = logging.getLogger('hl.plone.boardnotifications.subscribe')


class Subscriptions(PersistentMapping):

    """
    Part of this code has been taken from Products.PloneboardSubscription
    """

    implements(ISubscriptions)

    @staticmethod
    def key_for_obj(obj):
        return '/'.join(obj.getPhysicalPath()) 

    def subscribers_for(self, obj):
        key = self.key_for_obj(obj)
        subscriberids = self.get(key)
        if subscriberids is None:
            return []
        mtool = getToolByName(obj, 'portal_membership')
        return filter(lambda o: o is not None, [mtool.getMemberById(sid) for sid in subscriberids])

    def add(self, obj, user):
        """ Adds user """
        obj_id = self.key_for_obj(obj)
        if obj_id not in self.keys():
            self[obj_id] = PersistentList()
        if user not in self[obj_id]:
            log.info('Subscribing user %s to %s' % (user,  obj_id))
            self[obj_id].append(user)

    def check_subscriber(self, obj):
        """ Checks user """
        mtool = getToolByName(obj, 'portal_membership')
        user = mtool.getAuthenticatedMember().getId()
        return self.check_subscriber_id(self.key_for_obj(obj), user)

    def check_subscriber_id(self, obj_id, user):
        """ Checks user """
        if obj_id not in self.keys():
            return False
        return user in self[obj_id]

    def remove(self, obj, user):
        """ Deletes user """
        obj_id = self.key_for_obj(obj)
        if self.check_subscriber_id(obj_id, user):
            log.info('Removing subscription of user %s to %s' % (user, obj_id))
            self[obj_id].remove(user)
            if len(self[obj_id]) == 0:
                del self[obj_id]




import logging
import formatter
import StringIO
from lxml import html
from htmllib import HTMLParser
from persistent import Persistent
from persistent.mapping import PersistentMapping
from zope.component import adapts, getUtility, queryUtility
from zope.formlib.form import FormFields
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFDefault.formlib.schema import SchemaAdapterBase
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.interface import implements
from plone.app.controlpanel.form import ControlPanelForm
from Products.MailHost.interfaces import IMailHost
from email.parser import HeaderParser
from email.mime.text import MIMEText
from .interfaces import INotifierSchema, INotifier, ISubscriptions
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('hl.plone.boardnotifications')

log = logging.getLogger('hl.plone.boardnotifications.notify')


class NotifierControlPanelAdapter(SchemaAdapterBase):

    adapts(IPloneSiteRoot)
    implements(INotifierSchema)

    def get_subject(self):
        util = queryUtility(INotifier)
        return getattr(util, 'subject', '')

    def set_subject(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.subject = value

    subject = property(get_subject, set_subject)

    def get_signature(self):
        util = queryUtility(INotifier)
        return getattr(util, 'signature', '')

    def set_signature(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.signature = value

    signature = property(get_signature, set_signature)

    def get_salutations(self):
        util = queryUtility(INotifier)
        salutations = getattr(util, 'salutations', '')
        if salutations:
            salutations = '\n'.join([':'.join(item) for item in salutations.items()])
        return salutations

    def set_salutations(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            di = {}
            if value in (None, ''):
                di[''] = ''
            else:
                di.update([kv.split(':') for kv in value.split('\n')])
            util.salutations = di

    salutations = property(get_salutations, set_salutations)

    def get_comment_edited_text(self):
        util = queryUtility(INotifier)
        return getattr(util, 'comment_edited_text', '')

    def set_comment_edited_text(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.comment_edited_text = value

    comment_edited_text = property(get_comment_edited_text, set_comment_edited_text)

    def get_thread_moved_text(self):
        util = queryUtility(INotifier)
        return getattr(util, 'thread_moved_text', '')

    def set_thread_moved_text(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.thread_moved_text = value

    thread_moved_text = property(get_thread_moved_text, set_thread_moved_text)

    def get_comment_deleted_text(self):
        util = queryUtility(INotifier)
        return getattr(util, 'comment_deleted_text', '')

    def set_comment_deleted_text(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.comment_deleted_text = value

    comment_deleted_text = property(get_comment_deleted_text, set_comment_deleted_text)

    def get_subscription_comment_added_text(self):
        util = queryUtility(INotifier)
        return getattr(util, 'subscription_comment_added_text', '')

    def set_subscription_comment_added_text(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.subscription_comment_added_text = value

    subscription_comment_added_text = property(get_subscription_comment_added_text, set_subscription_comment_added_text)

    def get_subscription_comment_edited_text(self):
        util = queryUtility(INotifier)
        return getattr(util, 'subscription_comment_edited_text', '')

    def set_subscription_comment_edited_text(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.subscription_comment_edited_text = value

    subscription_comment_edited_text = property(get_subscription_comment_edited_text, set_subscription_comment_edited_text)

    def get_mailin_enabled(self):
        util = queryUtility(INotifier)
        return getattr(util, 'mailin_enabled', '')

    def set_mailin_enabled(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.mailin_enabled = value

    mailin_enabled = property(get_mailin_enabled, set_mailin_enabled)

    def get_fake_manager(self):
        util = queryUtility(INotifier)
        return getattr(util, 'fake_manager', '')

    def set_fake_manager(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.fake_manager = value

    fake_manager = property(get_fake_manager, set_fake_manager)

    def get_listen_addresses(self):
        util = queryUtility(INotifier)
        return getattr(util, 'listen_addresses', '')

    def set_listen_addresses(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.listen_addresses = value

    listen_addresses = property(get_listen_addresses, set_listen_addresses)

    def get_add_attachments(self):
        util = queryUtility(INotifier)
        return getattr(util, 'add_attachments', '')

    def set_add_attachments(self, value):
        util = queryUtility(INotifier)
        if util is not None:
            util.add_attachments = value

    add_attachments = property(get_add_attachments, set_add_attachments)


class NotifierControlPanel(ControlPanelForm):

    form_fields = FormFields(INotifierSchema)

    label = _(u'Settings for board notification mails')
    description = _(u'Here you can configure the mail texts for mails that are '
            'sent out to creators or subscribers of board content when it '
            'changes. You can use the following keywords to replace them using '
            'content coming from the thread: %(threadtitle)s, %(threadurl)s, '
            '%(boardtitle)s, %(mailsignature)s, %(salutation)s and, when '
            'appropriate, %(commenturl)s and %(commenttext)s.')
    form_name = u' Notifier Settings'

    def updateWidgets(self):
        super(NotifierControlPanel, self).updateWidgets()
        self.widgets['subject'].size = 30


class Notifier(Persistent):

    implements(INotifier)

    valid_headers = ('Subject', 'From', 'To')

    def __init__(self):
        self.subject = None
        self.signature = None
        self.comment_edited_text = None
        self.comment_deleted_text = None
        self.subscription_comment_added_text = None
        self.subscription_comment_edited_text = None
        self.thread_moved_text = None
        self._salutations = PersistentMapping()

    def create_plaintext_message(self, text):
        """ Create a plain-text-message by parsing the html
            and attaching links as endnotes

            Modified from EasyNewsletter/content/ENLIssue.py
        """
        # This reflows text which we don't want, but it creates
        # parser.anchorlist which we do want.
        textout = StringIO.StringIO()
        formtext = formatter.AbstractFormatter(formatter.DumbWriter(textout))
        parser = HTMLParser(formtext)
        parser.feed(text)
        parser.close()

        # append the anchorlist at the bottom of a message
        # to keep the message readable.
        counter = 0
        anchorlist = "\n\n" + '----' + "\n\n"
        for item in parser.anchorlist:
            counter += 1
            anchorlist += "[%d] %s\n" % (counter, item)

        # This reflows text:
        # text = textout.getvalue() + anchorlist
        # This just strips tags, no reflow
        text = html.fromstring(text).text_content()
        text += anchorlist
        del textout, formtext, parser, anchorlist
        return text

    def get_salutations(self):
        return self._salutations

    def set_salutations(self, mapping):
        if isinstance(mapping, PersistentMapping):
            self._salutations = mapping
        else:
            self._salutations = PersistentMapping()
            self._salutations.update(mapping)

    salutations = property(get_salutations, set_salutations)

    def _salutation_for_member(self, mdata):
        """
        answer an appropriate salutation
        """
        key = mdata.get('salutation', '')
        return self.salutations.get(key, '') % mdata

    @staticmethod
    def _encoding():
        return getUtility(ISiteRoot).getProperty('email_charset', 'utf-8')

    def _notify(self, mdata, text):
        headers = {}
        headers.update([tp for tp in HeaderParser().parsestr(text.encode(self._encoding())).items() if tp[0] in self.valid_headers])
        if headers.keys():
            text = '\n\n'.join(text.split('\n\n')[1:])
        text = self.create_plaintext_message(text)
        msg = MIMEText(text, _charset=self._encoding())
        msg['Subject'] = self.subject
        msg['From'] = getUtility(ISiteRoot).email_from_address
        msg['To'] = mdata.get('email')
        for k, v in headers.items():
            msg.replace_header(k, v)
        mh = getUtility(IMailHost)
        mh.send(msg)

    def _thread_info(self, thread):
        di = {}
        di['threadtitle'] = thread.Title().decode(self._encoding())
        di['threadurl'] = thread.absolute_url()
        di['boardtitle'] = thread.getForum().Title().decode(self._encoding())
        di['mailsignature'] = self.signature
        return di

    @classmethod
    def _memberdata_for_content(cls, content):
        return cls._memberdata_for(content.Creator())

    @classmethod
    def _memberdata_for(cls, memberid):
        site = getUtility(ISiteRoot)
        mtool = getToolByName(site, 'portal_membership')
        mdtool = getToolByName(site, 'portal_memberdata')
        keys = mdtool.propertyIds()
        mdata = mtool.getMemberById(memberid)
        if mdata is None: # no memberdata, most likely the user has been deleted
            return
        result = {}
        result.update([(k, str(mdata.getProperty(k)).decode(cls._encoding())) for k in keys])
        return result

    def comment_edited(self, comment):
        """
        A comment has been edited. Notify the creator of the comment.
        """
        # do not notify the creator if she has edited the comment herself
        mtool = getToolByName(comment, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        creator = mtool.getMemberById(comment.Creator())
        if (member == creator) or creator is None or not self.comment_edited_text or not self.comment_edited_text.strip():
            return
        thread = comment.getConversation()
        di = self._thread_info(thread)
        di.update(self._memberdata_for_content(comment))
        di['salutation'] = self._salutation_for_member(di)
        di['commenturl'] = comment.absolute_url()
        di['commenttext'] = comment.getText()
        self._notify(di, self.comment_edited_text % di)
        log.info('comment %s has been edited, notified owner %s' % (di['commenturl'], di.get('email')))

    def thread_moved(self, thread):
        """
        A thread has been moved to a new board. Notify all contributors.
        """
        if not self.thread_moved_text or not self.thread_moved_text.strip():
            return
        di = self._thread_info(thread)
        memberids = set([comment.Creator() for comment in thread.getComments()])
        for memberid in memberids:
            md = self._memberdata_for(memberid)
            if md is None:
                log.info('member with id %s could not be found, unable to send notification for %s' % (memberid, di['threadurl']))
                continue
            di.update(md)
            di['salutation'] = self._salutation_for_member(di)
            self._notify(di, self.thread_moved_text % di)
            log.info('thread %s has been moved, notified contributor %s' % (di['threadurl'], di.get('email')))

    def comment_deleted(self, comment):
        """
        A comment has been deleted. Notify its creator.
        """
        if not self.comment_deleted_text or not self.comment_deleted_text.strip():
            return
        thread = comment.getConversation()
        di = self._thread_info(thread)
        di['commenturl'] = comment.absolute_url()
        di['commenttext'] = comment.getText()
        md = self._memberdata_for_content(comment)
        if md is None:
            log.info('member with id %s could not be found, unable to send notification for %s' % (comment.Creator(), di['commenturl']))
            return
        di.update(self._memberdata_for_content(comment))
        di['salutation'] = self._salutation_for_member(di)
        self._notify(di, self.comment_deleted_text % di)
        log.info('comment %s has been deleted, notified owner %s' % (di['commenturl'], di.get('email')))

    def subscription_comment_edited(self, comment):
        """
        A comment has been edited. Notify thread subscribers.
        """
        if not self.subscription_comment_edited_text or not self.subscription_comment_edited_text.strip():
            return
        thread = comment.getConversation()
        forum = thread.getForum()
        di = self._thread_info(thread)
        di['commenturl'] = comment.absolute_url()
        di['commenttext'] = comment.getText()
        subscriptions = getUtility(ISubscriptions)
        subscribers = subscriptions.subscribers_for(thread)
        subscribers = set(subscriptions.subscribers_for(thread)) | set(subscriptions.subscribers_for(forum))
        mdtool = getToolByName(comment, 'portal_memberdata')
        keys = mdtool.propertyIds()
        for mdata in subscribers:
            if mdata.getId() == comment.Creator():
                continue
            di.update([(k, str(mdata.getProperty(k)).decode(self._encoding())) for k in keys])
            di['salutation'] = self._salutation_for_member(di)
            self._notify(di, self.subscription_comment_edited_text % di)
            log.info('comment %s has been edited, notified subscriber %s' % (di['commenturl'], di.get('email')))

    def subscription_comment_added(self, comment):
        """
        A comment has been added to a thread. Notify thread subscribers.
        """
        if not self.subscription_comment_added_text or not self.subscription_comment_added_text.strip():
            return
        thread = comment.getConversation()
        forum = thread.getForum()
        di = self._thread_info(thread)
        di['commenturl'] = comment.absolute_url()
        di['commenttext'] = comment.getText()
        subscriptions = getUtility(ISubscriptions)
        subscribers = set(subscriptions.subscribers_for(thread)) | set(subscriptions.subscribers_for(forum))
        mdtool = getToolByName(comment, 'portal_memberdata')
        keys = mdtool.propertyIds()
        for mdata in subscribers:
            if mdata.getId() == comment.Creator():
                continue
            di.update([(k, str(mdata.getProperty(k)).decode(self._encoding())) for k in keys])
            di['salutation'] = self._salutation_for_member(di)
            self._notify(di, self.subscription_comment_added_text % di)
            log.info('comment %s has been added, notified subscriber %s' % (di['commenturl'], di.get('email')))




from zope.component import getUtility
from hl.plone.boardnotifications.interfaces import ISubscriptions
from Products import Five
from Products.CMFCore.utils import getToolByName
from plone.app.layout.viewlets.common import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class Subscribe(Five.BrowserView):

    def __call__(self):
        mtool = getToolByName(self.context, 'portal_membership')
        subscriptions = getUtility(ISubscriptions)
        mem = mtool.getAuthenticatedMember().getId()
        subscriptions.add(self.context, mem)
        self.request.response.redirect(self.context.absolute_url() + '#subscribe')


class Unsubscribe(Five.BrowserView):

    def __call__(self):
        mtool = getToolByName(self.context, 'portal_membership')
        subscriptions = getUtility(ISubscriptions)
        mem = mtool.getAuthenticatedMember().getId()
        subscriptions.remove(self.context, mem)
        self.request.response.redirect(self.context.absolute_url() + '#subscribe')


class SubscriptionViewlet(ViewletBase):

    index = ViewPageTemplateFile('subscribe.pt')

    def is_subscribed_to_forum(self):
        subscriptions = getUtility(ISubscriptions)
        if self.context.portal_type == 'PloneboardForum':
            forum = self.context
        else:
            thread = self.context.getConversation(self.context.id)
            forum = thread.getForum()
        return subscriptions.check_subscriber(forum)

    def is_subscribed(self):
        subscriptions = getUtility(ISubscriptions)
        return subscriptions.check_subscriber(self.context)




# -*- coding: utf-8 -*-
import unittest
import transaction
from Testing import ZopeTestCase
from zope.component import getGlobalSiteManager, queryUtility
from zope.interface.verify import verifyObject
from Products.MailHost.interfaces import IMailHost
from Products.CMFCore.interfaces import IMembershipTool, IMemberDataTool, ISiteRoot
from mocks import MailHostMock, MembershipToolMock, MemberdataToolMock, MemberDataMock, ForumMock, ConversationMock, CommentMock
from hl.plone.boardnotifications.interfaces import INotifier


class NotifierTestLayer(ZopeTestCase.layer.ZopeLite):

    @classmethod
    def setUp(cls):
        gsm = getGlobalSiteManager()
        gsm.registerUtility(MailHostMock(), IMailHost)
        gsm.registerUtility(MembershipToolMock(), IMembershipTool)
        gsm.registerUtility(MemberdataToolMock(), IMemberDataTool)


class NotifierTests(unittest.TestCase):

    layer = NotifierTestLayer

    def setUp(self):
        transaction.begin()
        self.app = ZopeTestCase.app()
        gsm = getGlobalSiteManager()
        gsm.registerUtility(self.app, ISiteRoot)
        self.app.email_from_address = 'forum@lexware.de'
        mtool = queryUtility(IMembershipTool)
        self.app._setObject(mtool.id, mtool)
        mdtool = queryUtility(IMemberDataTool)
        self.app._setObject(mdtool.id, mdtool)
        md1 = MemberDataMock(id='123456',
                             email='max.mustermann@haufe-lexware.com',
                             salutation='Herr',
                             lastname='Mustermann',
                             firstname='Max')
        md2 = MemberDataMock(id='654321',
                             email='liese.mueller@haufe-lexware.com',
                             salutation='Frau',
                             lastname='M\xc3\xbcller',
                             firstname='Liese')
        md3 = MemberDataMock(id='123321',
                             email='nobody@haufe-lexware.com',
                             salutation='Frau',
                             lastname='Body',
                             firstname='No')
        mtool = queryUtility(IMembershipTool)
        mtool.members[md1['id']] = md1
        mtool.members[md2['id']] = md2
        mtool.members[md3['id']] = md3
        forum = ForumMock('testforum', 'test forum')
        thread = ConversationMock(id='testthread',
                                  title='test thread',
                                  forum=forum,
                                  creator=md1['id'],
                                  commenttext='Awesome first comment!')
        comment = CommentMock(id='testcomment',
                              title='Re: test',
                              conversation=thread,
                              text='Awesome second comment!',
                              creator=md2['id'])
        thread.comments.append(comment)
        thread._setObject(comment.id, comment)
        forum._setObject(thread.id, thread)
        self.app._setObject(forum.id, forum)

    def tearDown(self):
        transaction.abort()
        ZopeTestCase.close(self.app)

    def _make_one(self):
        from hl.plone.boardnotifications.notify import Notifier
        return Notifier()

    def _register_subscriptions(self):
        from hl.plone.boardnotifications.subscribe import Subscriptions
        from hl.plone.boardnotifications.interfaces import ISubscriptions
        gsm = getGlobalSiteManager()
        gsm.registerUtility(Subscriptions(), ISubscriptions)
        return queryUtility(ISubscriptions)

    def test_interface(self):
        verifyObject(INotifier, self._make_one())

    def test_thread_moved(self):
        n = self._make_one()
        n.thread_moved_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\nthreadurl:%(threadurl)s\nboardtitle:%(boardtitle)s\nsignature:%(mailsignature)s'
        n.signature='signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'}
        # if a user posts more than one comment, he still should get notified only once.
        comment = CommentMock(id='testcomment2',
                              title='Re: test',
                              conversation=self.app.testforum.testthread,
                              text='Awesome third comment!',
                              creator='123456')
        self.app.testforum.testthread.comments.append(comment)
        self.app.testforum.testthread._setObject(comment.id, comment)
        mh = queryUtility(IMailHost)
        mh.emails = []
        n.thread_moved(self.app.testforum.testthread)
        got = len(mh.emails)
        self.failUnless(got==2, 'notifier should have sent 2 emails, got %s instead' % got)
        for mail in mh.emails:
            got = {}
            got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
            self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
            self.failUnless(got['threadurl']=='http://nohost/testforum/testthread', 'unexpected thread url, got "%s"' % got['threadurl'])
            self.failUnless(got['boardtitle']=='test forum', 'unexpected board title, got "%s"' % got['boardtitle'])
            self.failUnless(got['signature']=='signature', 'unexpected signature')
            if mail[0][0].get('To') == 'max.mustermann@haufe-lexware.com':
                self.failUnless(got['salutation']=='Sehr geehrter Herr Max Mustermann', 'unexpected salutation, got "%s"' % got['salutation'])
            else:
                self.failUnless(got['salutation']=='Sehr geehrte Frau Liese M=C3=BCller', 'unexpected salutation, got "%s"' % got['salutation'])
        mh.emails = []
        n.thread_moved_text = None
        n.thread_moved(self.app.testforum.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is None')
        mh.emails = []
        n.thread_moved_text = ''
        n.thread_moved(self.app.testforum.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is empty')
        mh.emails = []
        n.thread_moved_text = ' \r\n '
        n.thread_moved(self.app.testforum.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template contains only whitespace')

    def test_thread_moved_missing_memberdata(self):
        n = self._make_one()
        n.thread_moved_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\nthreadurl:%(threadurl)s\nboardtitle:%(boardtitle)s\nsignature:%(mailsignature)s'
        n.signature='signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'}
        # if a user posts more than one comment, he still should get notified only once.
        comment = CommentMock(id='testcomment2',
                              title='Re: test',
                              conversation=self.app.testforum.testthread,
                              text='Awesome!',
                              creator='123456')
        self.app.testforum.testthread.comments.append(comment)
        self.app.testforum.testthread._setObject(comment.id, comment)
        mh = queryUtility(IMailHost)
        mh.emails = []
        mtool = queryUtility(IMembershipTool)
        mtool.members['654321'] = None
        n.thread_moved(self.app.testforum.testthread)
        got = len(mh.emails)
        self.failUnless(got==1, 'notifier should have sent 1 email, got %s instead' % got)
        mail = mh.emails[0]
        got = {}
        got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
        self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
        self.failUnless(got['threadurl']=='http://nohost/testforum/testthread', 'unexpected thread url, got "%s"' % got['threadurl'])
        self.failUnless(got['boardtitle']=='test forum', 'unexpected board title, got "%s"' % got['boardtitle'])
        self.failUnless(got['signature']=='signature', 'unexpected signature')
        self.failUnless(got['salutation']=='Sehr geehrter Herr Max Mustermann', 'unexpected salutation, got "%s"' % got['salutation'])

    def test_comment_edited(self):
        n = self._make_one()
        n.comment_edited_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\ncommenttext:%(commenttext)s\nsignature:%(mailsignature)s'
        n.signature = u'signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'} 
        n.comment_edited(self.app.testforum.testthread.testthread)
        mh = queryUtility(IMailHost)
        mail = mh.emails[-1]
        got = {}
        got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
        self.failUnless(got['salutation']=='Sehr geehrter Herr Max Mustermann', 'unexpected salutation, got "%s"' % got['salutation'])
        self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
        self.failUnless(got['commenturl']=='http://nohost/testforum/testthread/testthread', 'unexpected comment url, got "%s"' % got['commenturl'])
        self.failUnless(got['commenttext']=='Awesome first comment!', 'unexpected comment text, got "%s"' % got['commenttext'])
        mh.emails = []
        self.app.portal_membership.authenticated = '123456'
        n.comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)  
        self.failUnless(got==0, 'no mail should be sent to the creator of a comment if he edits it.')
        self.app.portal_membership.authenticated = None
        n.comment_edited_text = None
        n.comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is None')
        mh.emails = []
        n.comment_edited_text = ''
        n.comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is empty')
        mh.emails = []
        n.comment_edited_text = ' \r\n   '
        n.comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template contains only whitespace')

    def test_comment_edited_missing_memberdata(self):
        n = self._make_one()
        n.comment_edited_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\ncommenttext:%(commenttext)s\nsignature:%(mailsignature)s'
        n.signature = u'signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'} 
        mtool = queryUtility(IMembershipTool)
        mtool.members['123456'] = None
        mh = queryUtility(IMailHost)
        mh.emails = []
        n.comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'comment creator has been deleted - no mail should be sent')

    def test_comment_deleted(self):
        n = self._make_one() 
        n.comment_deleted_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\ncommenttext:%(commenttext)s\nsignature:%(mailsignature)s'
        n.signature='signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'} 
        n.comment_deleted(self.app.testforum.testthread.testcomment)
        mh = queryUtility(IMailHost)
        mail = mh.emails[-1]
        got = {}
        got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
        self.failUnless(got['salutation']=='Sehr geehrte Frau Liese M=C3=BCller', 'unexpected salutation, got "%s"' % got['salutation'])
        self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
        self.failUnless(got['commenturl']=='http://nohost/testforum/testthread/testcomment', 'unexpected comment url, got "%s"' % got['commenturl'])
        self.failUnless(got['commenttext']=='Awesome second comment!', 'unexpected comment text, got "%s"' % got['commenttext'])
        mh.emails = []
        n.comment_deleted_text = None
        n.comment_deleted(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is None')
        mh.emails = []
        n.comment_deleted_text = ''
        n.comment_deleted(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is empty')
        mh.emails = []
        n.comment_deleted_text = ' \r\n '
        n.comment_deleted(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template contains only whitespace')

    def test_comment_deleted_missing_memberdata(self):
        n = self._make_one()
        n.comment_deleted_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\ncommenttext:%(commenttext)s\nsignature:%(mailsignature)s'
        n.signature='signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'}
        mtool = queryUtility(IMembershipTool)
        mtool.members['654321'] = None
        mh = queryUtility(IMailHost)
        mh.emails = []
        n.comment_deleted(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'comment creator has been deleted - no mail should be sent')

    def test_subscription_comment_edited(self):
        n = self._make_one()
        subscriptions = self._register_subscriptions()
        # Subscribe to the forum
        subscriptions.add(self.app.testforum, '123321')
        # Subscribe to a thread
        subscriptions.add(self.app.testforum.testthread, '654321')
        n.subscription_comment_edited_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\ncommenttext:%(commenttext)s\nsignature:%(mailsignature)s'
        n.signature='signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'} 
        mh = queryUtility(IMailHost)
        mh.emails = []
        n.subscription_comment_edited(self.app.testforum.testthread.testthread)
        self.failUnless(len(mh.emails)==2, 'expected two mails for two subscribers')
        for mail in mh.emails:
            got = {}
            got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
            self.failUnless(got['salutation'] in ['Sehr geehrte Frau Liese M=C3=BCller', 'Sehr geehrte Frau No Body'], 'unexpected salutation, got "%s"' % got['salutation'])
            self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
            self.failUnless(got['commenturl']=='http://nohost/testforum/testthread/testthread', 'unexpected comment url, got "%s"' % got['commenturl'])
            self.failUnless(got['commenttext']=='Awesome first comment!', 'unexpected comment text, got "%s"' % got['commenttext'])
        comment = CommentMock(id='testcomment2',
                              title='Re: test',
                              conversation=self.app.testforum.testthread,
                              text='Awesome!',
                              creator='123456')
        self.app.testforum.testthread.comments.append(comment)
        self.app.testforum.testthread._setObject(comment.id, comment)
        subscriptions.add(self.app.testforum.testthread, '123456')
        mh.emails = []   
        n.subscription_comment_edited(self.app.testforum.testthread.testcomment2)
        self.failUnless(len(mh.emails)==2, 'expected two mails for two subscribers')
        for mail in mh.emails:
            mail = mh.emails[-1]
            got = {}
            got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
            self.failUnless(got['salutation'] in ['Sehr geehrte Frau Liese M=C3=BCller', 'Sehr geehrte Frau No Body'], 'unexpected salutation, got "%s"' % got['salutation'])
        mh.emails = []
        n.subscription_comment_edited_text = None
        n.subscription_comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is None')
        mh.emails = []
        n.subscription_comment_edited_text = ''
        n.subscription_comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is empty')
        mh.emails = []
        n.subscription_comment_edited_text = ' \r\n '
        n.subscription_comment_edited(self.app.testforum.testthread.testthread)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template contains only whitespace')

    def test_subscription_comment_added(self):
        n = self._make_one()
        subscriptions = self._register_subscriptions()
        # Subscribe to the forum
        subscriptions.add(self.app.testforum, '123321')
        # Subscribe to a thread
        subscriptions.add(self.app.testforum.testthread, '123456')
        n.subscription_comment_added_text = u'salutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\nsignature:%(mailsignature)s'
        n.signature='signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'} 
        mh = queryUtility(IMailHost)
        mh.emails = []
        n.subscription_comment_added(self.app.testforum.testthread.testcomment)
        for mail in mh.emails:
            got = {}
            got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
            self.failUnless(got['salutation'] in ['Sehr geehrter Herr Max Mustermann', 'Sehr geehrte Frau No Body'], 'unexpected salutation, got "%s"' % got['salutation'])
            self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
            self.failUnless(got['commenturl']=='http://nohost/testforum/testthread/testcomment', 'unexpected comment url, got "%s"' % got['commenturl'])
        # a user will not be notified if he is the author of the comment
        comment = CommentMock(id='testcomment2',
                              title='Re: test',
                              conversation=self.app.testforum.testthread,
                              text='Awesome!',
                              creator='654321')
        self.app.testforum.testthread.comments.append(comment)
        self.app.testforum.testthread._setObject(comment.id, comment)
        subscriptions.add(self.app.testforum.testthread, '654321')
        mh.emails = [] 
        n.subscription_comment_added(self.app.testforum.testthread.testcomment2)
        got = len(mh.emails)
        self.failUnless(got==2, 'expected two emails, got %s' % got)
        for mail in mh.emails:
            got = {}
            got.update([tuple(kv.split(':', 1)) for kv in mail[0][0].as_string().split('\n\n')[1].split('\n')])
            self.failUnless(got['salutation'] in ['Sehr geehrter Herr Max Mustermann', 'Sehr geehrte Frau No Body'], 'unexpected salutation, got "%s"' % got['salutation'])
        mh.emails = [] 
        n.subscription_comment_added_text = None
        n.subscription_comment_added(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is None')
        mh.emails = [] 
        n.subscription_comment_added_text = ''
        n.subscription_comment_added(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template is empty')
        mh.emails = [] 
        n.subscription_comment_added_text = ' \r\n\t '
        n.subscription_comment_added(self.app.testforum.testthread.testcomment)
        got = len(mh.emails)
        self.failUnless(got==0, 'no mails should be sent if the mail template contains only whitespace')

    def test_parse_email_headers(self):
        n = self._make_one()
        n.comment_edited_text = u'Subject:Changes in %(threadurl)s\n\nsalutation:%(salutation)s\nthreadtitle:%(threadtitle)s\ncommenturl:%(commenturl)s\nsignature:%(mailsignature)s'
        n.signature = u'signature'
        n.salutations = {u'Herr':u'Sehr geehrter Herr %(firstname)s %(lastname)s', u'Frau':u'Sehr geehrte Frau %(firstname)s %(lastname)s'}
        mh = queryUtility(IMailHost)
        mh.emails = []
        n.comment_edited(self.app.testforum.testthread.testthread)
        mail = mh.emails[-1]
        got = {}
        msg = mail[0][0]
        got.update([tuple(kv.split(':', 1)) for kv in msg.as_string().split('\n\n')[1].split('\n')])
        self.failUnless(msg['Subject']=='Changes in http://nohost/testforum/testthread', 'unexpected email subject, got "%s"' % msg['Subject'])
        self.failUnless(got['salutation']=='Sehr geehrter Herr Max Mustermann', 'unexpected salutation, got "%s"' % got['salutation'])
        self.failUnless(got['threadtitle']=='test thread', 'unexpected thread title, got "%s"' % got['threadtitle'])
        self.failUnless(got['commenturl']=='http://nohost/testforum/testthread/testthread', 'unexpected comment url, got "%s"' % got['commenturl'])
        self.failUnless(got['signature']=='signature', 'unexpected signature, got "%s"' % got['signature'])
        mh.emails = []


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(NotifierTests),
        ))

if __name__ == '__main__':
    from Products.GenericSetup.testing import run
    run(test_suite())




import re
import base64
import logging
import urlparse
from email.Errors import HeaderParseError
from email import message_from_string
try:
    from email import utils as email_utils
    email_utils  # pyflakes
except ImportError:
    # BBB Python 2.4
    from email import Utils as email_utils
from email import Header

from zExceptions import NotFound
from AccessControl import Unauthorized
from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.User import UnrestrictedUser
from OFS.Image import File
from zope.component import getUtility
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView

from hl.plone.boardnotifications.interfaces import INotifier

logger = logging.getLogger('hl.plone.boardnotifications.mailin')
URL_RE = re.compile(r'\b(([\w-]+://?|www[.])[^\s()<>]+(?:\([\w\d]+\)|([^[:punct:]\s]|/)))')

# Ripped bodily from poimail.py, thank you Maurits!


class Receiver(BrowserView):

    def __init__(self, context, request):
        super(Receiver, self).__init__(context, request)
        notifier = getUtility(INotifier)
        self.enabled = notifier.mailin_enabled
        self.listen_addresses = notifier.listen_addresses
        self.fake_manager = notifier.fake_manager
        self.add_attachments = notifier.add_attachments

    def __call__(self):
        if not self.enabled:
            logger.debug('received mail-in request, but mail-in not enabled')
            raise NotFound
        mail = self.request.get('Mail', '')
        mail = mail.strip()
        if not mail:
            msg = u'No mail found in request'
            logger.warn(msg)
            return msg
        message = message_from_string(mail)

        logger.debug('--------')
        logger.debug(mail)
        logger.debug('--------')
        logger.debug(message)
        from_addresses = self.get_addresses(message, 'From')
        to_addresses = self.get_addresses(message, 'To')
        if not from_addresses or not to_addresses:
            msg = u'No From or To address found in request'
            logger.warn(msg)
            return msg
        for from_name, from_address in from_addresses:
            if from_address:
                break
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        email_from_address = portal.getProperty('email_from_address')
        if from_address.lower() == email_from_address.lower():
            # This too easily means that a message sent by Poi ends up
            # being added as a reply on an issue that we have just
            # created.
            msg = u'Ignoring mail from portal email_from_address'
            logger.info(msg)
            return msg

        subject_line = message.get('Subject', '')
        subjects = []
        decoded = Header.decode_header(subject_line)
        for decoded_string, charset in decoded:
            if charset:
                decoded_string = decoded_string.decode(charset)
            subjects.append(decoded_string)
        subject = u' '.join(subjects)

        logger.debug("Forum at %s received mail from %r to %r with "
                     "subject %r", self.context.absolute_url(),
                     from_address, to_addresses, subject)
        text, mimetype = self.get_text_and_mimetype(message)
        if not text:
            text = "Warning: no text found in email"
            mimetype = 'text/plain'
            logger.warn(text)
        logger.debug('Got payload with mimetype %s from email.', mimetype)

        # Create an attachment from the complete email.
        attachment = File('email.eml', 'E-mail', mail)

        tags = self.get_tags(message)
        if tags:
            logger.debug("Determined tags: %r", tags)
        else:
            logger.debug("Could not determine tags.")

        # Store original security manager.
        sm = getSecurityManager()
        # Possibly switch to a different user.
        self.switch_user(from_address)

        if self.add_attachments:
            attachments = self.get_attachments(message)
            attachments = [File(filename, filename, data) for filename, data in attachments]
        else:
            attachments = [attachment]

        target = self.find_conversation_or_thread(subject, text, tags, message)
        if target is None:
            # We don't allow creating conversations from mail, only replies
            logger.info('Could not find something to reply to')
        else:
            try:
                self.add_response(target, from_address, subject, text, mimetype, attachments)
            except Unauthorized, exc:
                logger.error(u'Unauthorized to add response: %s', exc)
                return u'Unauthorized'
            logger.info('Added mail as response to target %s',
                        target.absolute_url())

        # Restore original security manager
        setSecurityManager(sm)
        return mail

    def switch_user(self, from_address):
        """Switch the user.

        This possibly does two things:

        1. Switch to the user that belongs to the given email address.

        2. Give the user the Manage role for the duration of this
           request.

        This view is normally used by the smtp2zope script (or
        something similar) on the local machine.  That script may
        submit anonymously.  That could mean the current user does not
        have enough permissions to submit an issue or add a response.
        So we elevate his privileges by giving him the Manager role.
        But when we do that, this means anonymous users could abuse
        this to submit through the web.  That is not good.  So we only
        elevate privileges when the request originates on the local
        computer.
        """
        sm = getSecurityManager()
        remote_address = self.request.get('HTTP_X_FORWARDED_FOR')
        if not remote_address:
            # Note that request.get('HTTP_something') always returns
            # at least an empty string, also when the key is not in
            # the request, so a default value would be ignored.
            remote_address = self.request.get('REMOTE_ADDR')
        if remote_address not in self.listen_addresses:
            return

        # First, see if we can get an existing user based on the From
        # address.
        pas = getToolByName(self.context, 'acl_users')
        users = pas.searchUsers(email=from_address)
        # Also try lowercase
        from_address = from_address.lower()
        if not users:
            users = pas.searchUsers(email=from_address)
        # If 'email' is not in the properties (say: ldap), we can get
        # far too many results; so we do a double check.  Also,
        # apparently ldap can leave '\r\n' at the end of the email
        # address, so we strip it.  And we compare lowercase.
        users = [user for user in users if user.get('email') and
                 user.get('email').strip().lower() == from_address]
        user = None
        switched = False
        if users:
            user_id = users[0]['userid']
            user = pas.getUserById(user_id)
            if user:
                switched = True
        if not user:
            user = sm.getUser()
            # Getting the user id can be tricky.
            if hasattr(user, 'name'):
                # Works for Anonymous Users
                user_id = user.name
            elif hasattr(user, 'getUserId'):
                # Plone users
                user_id = user.getUserId()
            elif hasattr(user, 'getId'):
                # Root zope users
                user_id = user.getId()
            else:
                # Right...
                return

        # See if this user already has the Manager role, otherwise add it.
        if self.fake_manager and not user.allowed(self.context, ('Manager', )):
            logger.debug("Faking Manager role for user %s", user_id)
            user = UnrestrictedUser(user_id, '', ['Manager'], '')
            faked = True
        else:
            faked = False
        # Now see if we changed something.
        if not (faked or switched):
            return
        newSecurityManager(self.request, user)
        if switched:
            logger.debug("Switched to user %s", user_id)
        if faked:
            logger.debug("Faking Manager role for user %s", user_id)

    def get_addresses(self, message, header_name):
        """Get addresses from the header_name.

        This is usually 'From' or 'To', but other headers may contain
        addresses too, so we allow all, unlike we used to do.

        We expect just one From address and one To address, but
        multiple addresses can also be checked.

        May easily be something ugly like this:
        =?utf-8?q?Portal_Administrator_?=<m.van.rees@zestsoftware.nl>

        From the Python docs:

        decode_header(header)

          Decode a message header value without converting charset.

          Returns a list of (decoded_string, charset) pairs containing
          each of the decoded parts of the header.  Charset is None
          for non-encoded parts of the header, otherwise a lower-case
          string containing the name of the character set specified in
          the encoded string.

          An email.Errors.HeaderParseError may be raised when certain
          decoding error occurs (e.g. a base64 decoding exception).
        """
        if not header_name:
            raise ValueError

        address = message.get(header_name, '')
        try:
            decoded = Header.decode_header(address)
        except HeaderParseError:
            logger.warn("Could not parse header %r", address)
            return []
        logger.debug('Decoded header: %r', decoded)
        for decoded_string, charset in decoded:
            if charset is not None:
                # Surely this is no email address but a name.
                continue
            if '@' not in decoded_string:
                continue

            return email_utils.getaddresses((decoded_string, ))
        return []

    def get_manager(self, message, tags):
        """Determine the responsible manager.

        A custom implementation could pick a manager based on the tags
        that have already been determined.
        """
        default = '(UNASSIGNED)'
        return default

    def get_tags(self, message):
        """Determine the tags that should be set for this post.

        You could add tags based on e.g. the To or From address.
        """
        # TODO: do forum posts have tags?
        return []

    def add_response(self, target, from_address, subject, text, mimetype, attachments):
        # TODO: add all the attachments at once
        # target.addComment(subject, text, creator=None, files=[attachment])
        # TODO: fix quick and dirty text formatting
        text = "<pre>%s</pre>" % text
        if target.meta_type == 'PloneboardConversation':
            target.addComment('answer via mail'+subject, text, creator=from_address, files=attachments)
        elif target.meta_type == 'PloneboardComment':
            target.addReply('answer via mail'+subject, text, creator=from_address, files=attachments)

    def find_conversation_or_thread(self, subject, text, tags, message):
        """Find a conversation or thread for which this email is a response.

        Match based on an URL in the mail.

        The message is passed in as argument as well, to make
        alternative schemes possible.
        """
        target = None
        for match in URL_RE.finditer(text):
            url = match.group(0)
            path = urlparse.urlparse(url).path
            try:
                obj = self.context.unrestrictedTraverse(path[1:])
            except AttributeError:
                # Skip bad URLs
                continue
            if obj.meta_type in ['PloneboardComment',
                    'PloneboardConversation']:
                if target is None:
                    target = obj
                elif (obj.meta_type == 'PloneboardComment' and
                        target.meta_type == 'PloneboardConversation'):
                    # More specific wins
                    target = obj
        return target

    def get_text_and_mimetype(self, message):
        """Get text and mimetype for the body of the response.

        The mimetype is not always needed, but it is good to know
        whether we have html or plain text.

        We prefer to get plain text.  Actually, getting the html from
        the email looks quite okay as long as we put it through the
        safe html transform.
        """
        payload = message.get_payload()
        if not message.is_multipart():
            mimetype = message.get_content_type()
            charset = message.get_content_charset()
            logger.info("Charset: %r", charset)
            if charset and charset != 'utf-8':
                # We only want to store unicode or ascii or utf-8 in
                # Plone.
                # Decode to unicode:
                payload = payload.decode(charset, 'replace')
                # Encode to utf-8:
                payload = payload.encode('utf-8', 'replace')
            return payload, mimetype
        for part in payload:
            if part.is_multipart():
                text, mimetype = self.get_text_and_mimetype(part)
            else:
                text, mimetype = self.part_to_text_and_mimetype(part)
            text = text.strip()
            # Might be empty?
            if text:
                return text, mimetype
        return '', 'text/plain'

    def part_to_text_and_mimetype(self, part):
        if part.get_content_type() == 'text/plain':
            return part.get_payload(decode=True), 'text/plain'
        tt = getToolByName(self.context, 'portal_transforms')
        if part.get_content_type() == 'text/html':
            mimetype = 'text/x-html-safe'
            safe = tt.convertTo(mimetype, part.get_payload(decode=True),
                                mimetype='text/html')
            # Poi responses fail on view when you have the x-html-safe
            # mime type.  Fixed in Poi 1.2.12 (unreleased) but hey, we
            # only need that safe mimetype for the conversion.
            mimetype = 'text/html'
        else:
            # This might not work in all cases, e.g. for attachments,
            # but that is not tested yet.
            mimetype = 'text/plain'
            safe = tt.convertTo(mimetype, part.get_payload(decode=True))
        if safe is None:
            logger.warn("Converting part to mimetype %s failed.", mimetype)
            return u'', 'text/plain'
        return safe.getData(), mimetype

    def get_attachments(self, message):
        """Get attachments.
        """
        payload = message.get_payload()
        if not message.is_multipart():
            mimetype = message.get_content_type()
            if mimetype.startswith('text'):
                return []
            filename = message.get_filename()
            if not filename:
                return []
            encoding = message.get('Content-Transfer-Encoding', '')
            if encoding == 'base64':
                data = base64.decodestring(payload)
            elif encoding == 'binary':
                # Untested.
                data = payload
            else:
                # TODO: support other encodings?  Not sure if this
                # makes sense for anything else.
                return []
            return [(filename, data)]
        attachments = []
        for part in payload:
            attachments.extend(self.get_attachments(part))
        return attachments

    def create_post(self, **kwargs):
        """ Create a post in the given board, and perform workflow and
        rename-after-creation initialisation.
        """
        # We only allow replies to notification, at least for now.
        pass




import unittest
import transaction
from Testing import ZopeTestCase
from zope.component import getGlobalSiteManager
from zope.interface.verify import verifyObject
from Products.CMFCore.interfaces import ISiteRoot
from mocks import MembershipToolMock, ForumMock, ConversationMock, MemberDataMock
from hl.plone.boardnotifications.interfaces import ISubscriptions


class SubscriptionTests(unittest.TestCase):

    def _make_one(self):
        from hl.plone.boardnotifications.subscribe import Subscriptions
        subscriptions = Subscriptions()
        getGlobalSiteManager().registerUtility(subscriptions, ISubscriptions)
        return subscriptions

    def setUp(self):
        transaction.begin()
        self.app = ZopeTestCase.app()
        gsm = getGlobalSiteManager()
        gsm.registerUtility(self.app, ISiteRoot)
        self.app.email_from_address = 'forum@lexware.de'
        mtool = MembershipToolMock()
        member = MemberDataMock(id='123456',
                                email='max.mustermann@haufe-lexware.com',
                                salutation='Herr',
                                lastname='Mustermann',
                                firstname='Max')
        mtool.members[member['id']] = member
        self.app._setObject(mtool.id, mtool)
        forum = ForumMock('testforum', 'test forum')
        thread = ConversationMock(id='testthread',
                                  title='test thread',
                                  forum=forum,
                                  creator=member['id'])
        forum._setObject(thread.id, thread)
        self.app._setObject(forum.id, forum)

    def test_interface(self):
        verifyObject(ISubscriptions, self._make_one())

    def test_subscribers_for(self):
        subscriptions = self._make_one()
        thread = self.app.testforum.testthread
        subscriptions.add(thread, '123456')
        subscriptions.add(thread, '999999') # users that do not exist should be eliminated
        got = subscriptions.subscribers_for(thread)
        self.failUnless(len(got) == 1, 'expected one subscriber')
        email = got[0].getProperty('email')
        self.failUnless(email == 'max.mustermann@haufe-lexware.com', 'wrong memberdata %s' % got[0])


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SubscriptionTests),
        ))

if __name__ == '__main__':
    from Products.GenericSetup.testing import run
    run(test_suite())



from zope.component import adapter, queryUtility
from zope.lifecycleevent.interfaces import (
        IObjectAddedEvent,
        IObjectModifiedEvent,
        IObjectMovedEvent,
        IObjectRemovedEvent,
        )
from Products.Archetypes.interfaces import (
        IObjectEditedEvent,
        IObjectInitializedEvent,
        )
from Products.Ploneboard.interfaces import IConversation, IComment
from .notify import INotifier


@adapter(IConversation, IObjectMovedEvent)
def threadmoved(conv, event):
    """
    Send email notification when a thread was moved
    """
    if IObjectRemovedEvent.providedBy(event) or IObjectAddedEvent.providedBy(event):
        return
    n = queryUtility(INotifier)
    n.thread_moved(conv)

@adapter(IComment, IObjectModifiedEvent)
def commentedited(comment, event):
    """
    Send email notification when a comment has been edited
    """
    n = queryUtility(INotifier)
    n.comment_edited(comment)

@adapter(IComment, IObjectEditedEvent)
def subscriptioncommentedited(comment, event):
    """
    Send email notification to thread subscribers when a comment has been edited
    """
    n = queryUtility(INotifier)
    n.subscription_comment_edited(comment)

@adapter(IComment, IObjectInitializedEvent)
def subscriptioncommentadded(comment, event):
    """
    Send email notification to thread subscribers when a comment has been added
    """
    n = queryUtility(INotifier)
    # XXX imho the following call is missing in PloneboardComment.addReply, s. http://plone.org/products/ploneboard/issues/240
    comment.unmarkCreationFlag()
    n.subscription_comment_added(comment)

@adapter(IComment, IObjectRemovedEvent)
def commentdeleted(comment, event):
    """
    Send email notification when a comment has been edited
    """
    n = queryUtility(INotifier)
    n.comment_deleted(comment)




from setuptools import setup, find_packages
import os

version = '0.7'

setup(name='hl.plone.boardnotifications',
      version=version,
      description="Notify users when Ploneboard threads or comments are modified",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='Plone Ploneboard board forum notifications',
      author='see docs/CREDITS.txt',
      author_email='thomas.schorr@haufe-lexware.com',
      url='https://github.com/Haufe-Lexware/hl.plone.boardnotifications',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['hl', 'hl.plone'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.Ploneboard >= 3.0',
          # -*- Extra requirements: -*-
      ],
      extras_require={
          'mailin': [
              'smtp2zope',
              ]
          },
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from zope.interface import Interface
from zope.schema import Bool, BytesLine, List, Text, TextLine
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('hl.plone.boardnotifications')


class INotifier(Interface):

    """
    Notify in case of changes to comments/threads
    """

    def comment_edited(comment):
        """
        A comment has been edited. Notify the creator of the comment.
        """

    def thread_moved(thread):
        """
        A thread has been moved to a new board. Notify all contributors.
        """

    def comment_deleted(comment):
        """
        A comment has been deleted. Notify its creator.
        """

    def subscription_comment_edited(comment):
        """
        A comment has been edited. Notify thread subsribers.
        """

    def subscription_comment_added(comment):
        """
        A comment has been added to a thread. Notify thread subscribers.
        """


class ISubscriptions(Interface):

    """
    manage notification subscriptions for threads
    """

    def add(obj, user):
        """
        subscribe user to obj
        """

    def remove(obj, user):
        """
        remove user's subscription from obj
        """

    def check_subscriber(obj):
        """
        check wether the current user is subscribed to obj
        """

    def subscribers_for(obj):
        """
        answer all users that are subscribed to obj
        """


class INotifierSchema(Interface):

    """
    schema for the notifiers configuration
    """

    subject = TextLine(title=_(u'Subject'),
            description=_(u'Mail subject'))

    signature = Text(title=_(u'Signature'),
            description=_(u'Mail signature'))

    salutations = Text(title=_(u'Map Salutations'),
            required=False,
            description=_(u'Map salutation from memberdata property to email '
                'salutation (one mapping per line, use : to separate key and '
                'value).'))

    comment_edited_text = Text(title=_(u'Comment edited'),
            required=False,
            description=_(u'Mail text when a comment has been edited'))

    thread_moved_text = Text(title=_(u'Thread moved'),
            required=False,
            description=_(u'Mail text when a thread has been moved to another board'))

    comment_deleted_text = Text(title=_(u'Comment deleted'),
            required=False,
            description=_(u'Mail text when a comment has been deleted'))

    subscription_comment_edited_text = Text(title=_(u'Comment edited (Subscriber)'),
            required=False,
            description=_(u'Mail text to send to subscribers of a thread where '
                'a comment has been edited'))

    subscription_comment_added_text = Text(title=_(u'Comment added (Subscriber)'),
            required=False,
            description=_(u'Mail text to send to subscribers of a thread where '
                'a comment has been added'))

    mailin_enabled = Bool(title=_(u'Enable mail-in support'), 
            default=False,
            description=_(u'Check to enable mail-in support (needs smtp2zope).'))

    fake_manager = Bool(title=_(u'Mail-in: Fake Manager role?'), 
            default=True,
            description=_(u'Use Manager role during mail-in request.'))

    listen_addresses = List(title=_(u'Addresses to listen for mail'),
        description = _(u'Specify network addresses from which mail-in requests can originate, one by line.'),
        value_type = BytesLine(), 
        default = [u'127.0.0.1',], 
        required = False) 

    add_attachments = Bool(title=_(u'Mail-in: Add attachments?'), 
            default=True,
            description=_(u'Check to allow uploading of attachments during mail-in.'))

